package de.hazewood.rylius.mlgj1;

import de.hazewood.rylius.mlgj1.levels.Level;
import org.newdawn.slick.*;
import org.newdawn.slick.geom.Vector2f;

public abstract class Element {

    protected final Vector2f pos;
    protected final Vector2f vel;
    protected Image image;
    protected boolean removeable;
    protected Level level;
    protected Vector2f gravity;
    protected Sound pickupSound;
    private boolean pickable;

    public Element(Image image) {
        this(image, false);
    }

    public Element(Image image, boolean pickable) {
        this.image = image;
        this.pickable = pickable;
        pos = new Vector2f();
        vel = new Vector2f();
        gravity = new Vector2f();
    }

    public void init(Level level) {
        this.level = level;
    }

    public void update(GameContainer gc, int delta) {
        vel.add(gravity);
        pos.add(vel);
    }

    public void draw(GameContainer gc, Graphics g) {
        if (image != null) {
            image.draw(pos.getX(), pos.getY());
        }
    }

    public abstract boolean combine(Element other);

    public boolean hovered(int mx, int my) {
        return image != null && mx > pos.getX() && mx < pos.getX() + image.getWidth() && my > pos.getY() && my < pos.getY() + image.getHeight() && image.getColor((int) (mx - pos.getX()), (int) (my - pos.getY())).a > 0;
    }

    public boolean clicked(Input in) {
        return hovered(in.getMouseX(), in.getMouseY()) && in.isMousePressed(Input.MOUSE_LEFT_BUTTON);
    }

    public final float distanceTo(Vector2f pos) {
        return (float) Math.sqrt(Math.pow(pos.getX() - getPos().getX(), 2) + Math.pow(pos.getY() - getPos().getY(), 2));
    }

    public Vector2f origin() {
        if (image != null) {
            return new Vector2f(pos.getX() + image.getWidth() / 2, pos.getY() + image.getHeight() / 2);
        } else {
            return new Vector2f(getPos());
        }
    }

    public final float angleTo(Element ent) {
        return angleTo(ent.origin());
    }

    public final float angleTo(Vector2f vec) {
        return angleTo(vec.getX(), vec.getY());
    }

    public final float angleTo(float x, float y) {
        Vector2f origin = origin();
        float dx = origin.getX() - x;
        float dy = origin.getY() - y;
        return (float) Math.toDegrees(Math.atan2(dy, dx)) - 90;
    }

    public final Vector2f velocityTo(Vector2f dest, float speed) {
        return velocityTo(dest.getX(), dest.getY(), speed);
    }

    public final Vector2f velocityTo(float x, float y, float speed) {
        float vx, vy, dx, dy, alpha;
        Vector2f v = new Vector2f();
        Vector2f origin = origin();
        dx = origin.getX() - x;
        dy = origin.getY() - y;
        alpha = (float) Math.atan((dx / dy));
        vx = (float) -(Math.sin(alpha) * speed);
        vy = (float) -(Math.cos(alpha) * speed);
        if ((dx > 0 && vx > 0) || (dx < 0 && vx < 0)) {
            vx *= -1;
        }
        if ((dy > 0 && vy > 0) || (dy < 0 && vy < 0)) {
            vy *= -1;
        }
        v.set(vx, vy);
        return v;
    }

    public final void setGravity(Vector2f gravity) {
        this.gravity = gravity;
    }

    public final Vector2f getPos() {
        return pos;
    }

    public final Vector2f getVel() {
        return vel;
    }

    public boolean isRemoveable() {
        return removeable;
    }

    public boolean isPickable() {
        return pickable;
    }

    public Sound getPickupSound() {
        return pickupSound;
    }
}
