package de.hazewood.rylius.mlgj1.levels;

import de.hazewood.rylius.mlgj1.Element;
import de.hazewood.rylius.mlgj1.Game;
import de.hazewood.rylius.mlgj1.MLGJ1;
import de.hazewood.rylius.mlgj1.elements.chars.Discord;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.Image;
import org.newdawn.slick.Sound;
import org.newdawn.slick.geom.Vector2f;

import java.util.LinkedList;
import java.util.List;

public abstract class Level {

    protected final LinkedList<Element> elements;
    protected MLGJ1 main;
    protected Image backdrop;
    protected Element selected;
    protected Vector2f selectedPos;
    protected Discord discord;
    protected boolean finished;
    protected Sound pickupSound;
    protected Image win;
    protected Image loss;
    private final LinkedList<Event> events;
    private int time;

    public Level(MLGJ1 main, Image backdrop, int time) {
        this.main = main;
        this.backdrop = backdrop;
        this.time = time;

        elements = new LinkedList<Element>();
        events = new LinkedList<Event>();

        win = Game.image("level_win");
        loss = Game.image("level_loss");

        finished = false;

        pickupSound = Game.sound("pick");
    }

    public final void add(Element el) {
        el.init(this);
        elements.add(el);
    }

    public final void addEvent(String name, Element source, Object[] params) {
        events.add(new Event(name, source, params));
    }

    public final void update(GameContainer gc, int delta) {
        time -= delta;

        for (Event ev : events) {
            onEvent(ev);
        }
        events.clear();

        if (selected != null && selected.isRemoveable()) {
            selected = null;
            selectedPos = null;
        }

        List<Element> remove = new LinkedList<Element>();
        for (Element el : elements) {
            el.update(gc, delta);
            if (el.isRemoveable()) {
                remove.add(el);
            }
        }
        elements.removeAll(remove);

        updateInternal(gc, delta);
    }

    public final void draw(GameContainer gc, Graphics g) {
        backdrop.draw();

        for (Element el : elements) {
            el.draw(gc, g);
        }

        drawInternal(gc, g);
    }

    protected void drawRemainingTime(Graphics g) {
        if (!finished) {
            int s = remainingTime() / 1000;
            g.drawString("Time left: " + s + " second" + (s == 1 ? "" : "s"), 600, 9);
        }
    }

    public void pick(Element el) {
        if (finished || selected == el || discord == null) {
            return;
        }

        if (selected != null) {
            if (selected.combine(el)) {
                pick(null);
                return;
            }
            selected.getPos().set(selectedPos);
        }

        selected = el;
        if (selected != null) {
            if (selected.getPickupSound() != null) {
                selected.getPickupSound().play();
            }

            selectedPos = selected.getPos().copy();
            selected.getPos().set(discord.getPos().copy().add(discord.getItemPos()));
        }
    }

    public int remainingTime() {
        return time;
    }

    public boolean isFinished() {
        return finished;
    }

    public Sound getPickupSound() {
        return pickupSound;
    }

    protected abstract void onEvent(Event ev);

    protected abstract void updateInternal(GameContainer gc, int delta);

    protected abstract void drawInternal(GameContainer gc, Graphics g);

    static class Event {
        private String name;
        private Element source;
        private Object[] params;

        Event(String name, Element source, Object[] params) {
            this.name = name;
            this.source = source;
            this.params = params;
        }

        String getName() {
            return name;
        }

        Element getSource() {
            return source;
        }

        Object[] getParams() {
            return params;
        }
    }

}
